/**
 * Implements an eq control point
 */
class ieqcontrol extends iDraggable {

    constructor(options) {
        super(options);
        this._domElement.style.position = "absolute";
        var ids = options.paramData.id.split(",");
        this._freqParamId = ids[0];
        this._gainParamId = ids[1];
        this._qParamId = ids[2];
        this._bypassParamId = ids[3];
        this._lowerXPxLimit = 0;
        this._upperXPxLimit = 0;
        this._lowerYPxLimit = 0;
        this._upperYPxLimit = 0;
        this._currentWindowWidth = window.innerWidth;
        this._currentWindowHeight = window.innerHeight
        
        // set param
        this._controls = { 
            "freq": new iControl({'id':options.id, 'paramData':{'id': this._freqParamId}}),
            "gain": new iControl({'id':options.id, 'paramData':{'id': this._gainParamId}}),
            "q": new iControl({'id':options.id, 'paramData':{'id': this._qParamId}}),
            "bypass": new iControl({'id':options.id, 'paramData':{'id': this._bypassParamId}}),
        };
        AddControl(this._controls.freq);
        AddControl(this._controls.gain);
        AddControl(this._controls.q);
        AddControl(this._controls.bypass);
        
        document.removeEventListener("touchmove", this.touchMouseMove);
        document.removeEventListener("mousemove", this.touchMouseMove);
        
        // add event on the mouse wheel to change Q
        this.adjustQ = (event) => {
            this._controls.q.setValue(this._controls.q.getValue() + (event.deltaY / 100 / this._gearing));
        }
        this._domElement.addEventListener("wheel", this.adjustQ);
        
        this.touchMouseMove = (event) => {
      
          var clientX = -1;
          var clientY = -1;
          
          if(this._captured == true) {
            if(event.type == "touchmove") {
              var touch = event.changedTouches[0];
              clientX = touch.clientX;
              clientY = touch.clientY;
            } else if (event.buttons == 1) {
              if(this._captured == true) {
                event.preventDefault();
              }
              clientX = event.clientX;
              clientY = event.clientY;
            }
          
            this._endX = clientX;
            this._endY = clientY;
            
            var deltaX = ((this._endX - this._startX) / this._width) * 0.5 / this._gearing / 3;
            var deltaY = ((this._startY - this._endY) / this._height) * 0.5 / this._gearing;
            //this.setValue(this._value + delta);
            this._controls.freq.setValue(this._controls.freq.getValue() + deltaX);
            this._controls.gain.setValue(this._controls.gain.getValue() + deltaY);
            this._startX = clientX;
            this._startY = clientY;
            
          }
          
        };
        
        // remove the four dblclick event installed by the four child icontrols
        this._domElement.removeEventListener("dblclick", this.dblClickHandler, false);
        this._controls.freq._domElement.removeEventListener("dblclick", this._controls.freq.dblClickHandler, false);
        this._controls.gain._domElement.removeEventListener("dblclick", this._controls.gain.dblClickHandler, false);
        this._controls.q._domElement.removeEventListener("dblclick", this._controls.q.dblClickHandler, false);
        this._controls.bypass._domElement.removeEventListener("dblclick", this._controls.bypass.dblClickHandler);
        this._domElement.addEventListener("dblclick", event => {
            this._controls.bypass.setValue(1-Number(this._controls.bypass.getValue()));
        });
        
        this._domElement.addEventListener("mouseover", event => {
            if(!this._captured) {
                this._domElement.style.cursor = "move";
            }  
        });
                         
        document.addEventListener("touchmove", this.touchMouseMove);
        document.addEventListener("mousemove", this.touchMouseMove);
        
        this._controls.freq.setChangeCallback(function(value){
            this.updateDot();
        }.bind(this));
        
        this._controls.gain.setChangeCallback(function(value){
            this.updateDot();
        }.bind(this));
        
        this._controls.bypass.setChangeCallback(function(value){
            if (value < 0.5) {
                this._domElement.classList.remove('bypass');
            } else {
                this._domElement.classList.add('bypass');
            }
        }.bind(this));
        
        window.addEventListener("resize", (event) => {
            var ratio = window.innerWidth / this._currentWindowWidth;
            this._lowerXPxLimit *= ratio;
            this._upperXPxLimit *= ratio;
            this._lowerYPxLimit *= ratio;
            this._upperYPxLimit *= ratio;
            this._currentWindowHeight = window.innerHeight;
            this._currentWindowWidth = window.innerWidth;
            this.updateDot();
        });

    }
    
    setXAxisPxLimits(lower,upper) {
        this._lowerXPxLimit = Number(lower);
        this._upperXPxLimit = Number(upper);
    }
    
    setYAxisPxLimits(lower, upper) {
        this._lowerYPxLimit = Number(lower);
        this._upperYPxLimit = Number(upper);
    }
    
    updateDot() {
        var freqTranslate = this._lowerXPxLimit - (this._domElement.offsetWidth*0.5) + (this._controls.freq.getValue() * (this._upperXPxLimit - this._lowerXPxLimit));
        var gainTranslate = this._upperYPxLimit - (this._domElement.offsetHeight*0.5) + ((1 - this._controls.gain.getValue()) * (this._lowerYPxLimit - this._upperYPxLimit));
        this._domElement.style.left = Math.round(freqTranslate);
        this._domElement.style.top = Math.round(gainTranslate);
        //this._domElement.style.transform = "translate(" + Math.round(freqTranslate) + "px, -" + Math.round(gainTranslate) + "px)";
    }
    
}
